package common.msg;

import java.util.Properties;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import common.constant.SysConst;
import common.exception.OtherAppException;

/**
 * タイトル: メッセージユーティリティクラス<br>
 * 説明　　: 定義ファイルからのメッセージ読込み及びマップからのメッセージ取得を行う。
 * NAME: 石原英幸
 */
public class MsgUtilStatic {
	 /**  メッセージプロパティ */
	 static private Properties msgprop = null ;
	 /** 付加情報マーク */
	 static private final String mark = "@@";

	  /**
	   * モジュール情報出力フラグ
	   * */
	  private static boolean moduleNameFlg = true ;
	  /**
	   * タイムスタンプ書式
	   */
	  private static String MODINFO_FORMAT = "[$CLSNAME::$METHOD_NAME:$LINE_NUMBER]" ;

	/**
	 * コンストラクタ。<pre>
	 * 【説明】
	 * 何もしない。
	 * </pre>
	 *
	 */
    private MsgUtilStatic() {
    }

    static public void setMsgprop(Properties prop){
    	msgprop = prop ;
    }
	/**
	 * メッセージ取得。<pre>
	 * 【説明】
	 * 指定されたメッセージIDに対応したメッセージの取得及び付加情報の編集を行う。
	 * </pre>
	 * @param resourceType メッセージテキストリソース種別  1:マップ, 2:プロパティファイル
	 * @param msgid　メッセージID
	 * @param addmsg　付加情報
	 * @return　編集したメッセージ
	 * @throws OtherAppException
	 *
	 */
    static public String getMsg(String msgid, String[] addmsg) {
    	String defname = SysConst.MSG_MAP_CONFIG ;

    	//メッセージプロパティ から取得
        return getMsg(2, defname, msgid, addmsg);
    }
    static public String getMsg(int resourceType, String msgid, String[] addmsg) {
        return getMsg(resourceType, "", msgid, addmsg);
    }
    static public String getMsg(int resourceType, String defname, String msgid, String[] addmsg) {

        //編集元メッセージテキストを取得
        String src = "";
        switch(resourceType){
        case 1:
            src = getMapMsg(msgid);
            break;
        case 2:
			src = getPropMsg(msgid);
            break;
        case 3:
//			src = getMapSetMsg(defname, msgid);
            break;

        }

        if (src == null) {
            //当該IDのメッセージが無い
            return src;
        }

        if (addmsg == null) {
            return src;
        }
		if (addmsg.length == 0){
			return src;
		}
        StringBuffer sbf = new StringBuffer(src);

        for(int i=0; i < addmsg.length ; i++) {
            int idx = sbf.toString().indexOf(mark);

            if (idx == -1) {
                break; //マークが無いので終了
            }
            String addtxt = addmsg[i] ;
            if (addtxt.length() == 0){
            	addtxt = "[空文字]";
            }
			sbf = sbf.replace(idx, idx + mark.length(), addtxt);
        }
		String txt = sbf.toString();

//		// 末尾の"@@" は、取り除く
//		if (txt.endsWith(mark)){
//			txt.replace(mark,  "");
//		}

        return txt;
    }
    static private String getMapMsg(String msgid){
        String src = "";

        for(int i=0; i < MsgConst.msgmap.length; i++){
        	src = (String)MsgConst.msgmap[i].get(msgid);
        	if (src != null){
        		break;
        	}
        }
        return src ;
    }
    /**
     *
     * <pre>
     * 【概要】
     * </pre>
     * @param msgid
     * @return
     */
    static private String getPropMsg(String msgid) {

        String src = (String)msgprop.get(msgid);

        return src ;
    }
    /**
     * [メソッド名] メッセージ出力
     * <pre>
     * 説明：メッセージ出力を行う
     * </pre>
     * @param lvl ログの出力レベル
     * @param msg メッセージテキスト
     */
    static public void msgout(Level lvl, String msg ){

      Logger msgLogger = null ;

      String moduleInfo = "" ;

        /*モジュール情報出力フラグがtrue？*/
        if (moduleNameFlg){
          Throwable t = new Throwable();
          /*スタックトレースを取得*/
          StackTraceElement[] ste = t.getStackTrace();

          /*クラス名*/
          String className = "";
          /*メソッド名*/
          String methodName = "";
          /*行番号**/
          String lineNumber = "";

          for(int i=1 ; i < 3; i++){
            /*クラス名をスタックトレースから取得*/
            className = ste[i].getClassName();

            /*メソッド名をスタックトレースから取得*/
            methodName = ste[i].getMethodName();

            /*行番号をスタックトレースから取得*/
            lineNumber = String.valueOf(ste[i].getLineNumber());

            /*クラス名がメッセージ出力クラス名とイコールではない？*/
            if (!className.equals(MsgUtilStatic.class.getName())){
              break;
            }
          }

          /*タイムスタンプ書式を設定*/
          moduleInfo = MODINFO_FORMAT ;

          /*タイムスタンプのクラス名をスタックトレースから取得したクラス名で置換する*/
          moduleInfo = moduleInfo.replace("$CLSNAME", className);

          /*タイムスタンプのメソッド名をスタックトレースから取得したメソッド名で置換する*/
          moduleInfo = moduleInfo.replace("$METHOD_NAME", methodName);

          /*行番号がnullでなく、かつ、長さが0である？*/
          if ((lineNumber != null) && (lineNumber.length() > 0)){
            /*スタックトレースから取得した行番号で置換する*/
            moduleInfo = moduleInfo.replace("$LINE_NUMBER", lineNumber);

          }

          /*他カテゴリのログレベルの変更*/
          msgLogger = LogManager.getLogger(className);
        }


        /*メッセージにモジュール情報*/
        msg = moduleInfo + msg ;
        // メッセージ出力
        msgLogger.log(lvl, msg);

//        /*出力レベルがERROR？*/
//        if (lvl == Level.DEBUG){
//          /*メッセージを出力*/
//          msgLogger.debug( msg );
//        }
//        /*出力レベルがTRACE？*/
//        else if (lvl == Level.TRACE){
//          /*メッセージを出力*/
//          msgLogger.trace( msg );
//        }
//        /*出力レベルがINFO？*/
//        else if (lvl == Level.INFO){
//          /*メッセージを出力*/
//          msgLogger.info( msg );
//        }
//        /*出力レベルがWARN？*/
//        else if (lvl == Level.WARN){
//          /*メッセージを出力*/
//          msgLogger.warn( msg );
//        }
//        /*出力レベルがERROR？*/
//        else if (lvl == Level.ERROR){
//          /*メッセージを出力*/
//          msgLogger.error( msg );
//        }
//        /*出力レベルがFATAL？*/
//        else if (lvl == Level.FATAL){
//          msgLogger.fatal( msg );
//        }

    }
    /**
     * [メソッド名]例外情報編集
     * <pre>
     * 説明：例外情報編集を行う
     *</pre>
     * @param ex 例外オブジェクト
     * @return 編集したテキスト
     */
    static public String editExceptionMsg(Exception ex){

      StringBuilder sbf = new StringBuilder();

      /*クラス名とメソッド名、例外情報を追加する*/
      sbf.append(ex.getClass().getName()).append(" : ");
      sbf.append(ex.getMessage()).append(SysConst.CRLF);

      /*スタックトレースを取得する*/
      StackTraceElement[] traces = ex.getStackTrace();

      /*スタックトレース行数分ループ*/
      for(int i=1; i < traces.length; i++){
        /*スタックトレースを追加*/
        sbf.append(traces[i]).append(SysConst.CRLF);
      }
      return sbf.toString();
    }

}
